#include <cmath>
#include <fstream>
#include <iostream>

/**
 * This is a simple program that generates a 176 x 144, polarized 
 * "division by focal plane" pixel database file for use by the DIRSIG4
 * data-driven detector array model.  These types of focal planes use a
 * checkerboard pattern of linear polarization filters overlaid on the
 * array.  A pixel database file includes the pixel line-of-sight (LOS)
 * vectors (as an X/Y angle pair) and optional spectral response
 * information.  The program creates the pixel databse using a minimal
 * set of inputs including the focal length, pixel pitch and number of
 * pixels in each dimension.  The polarization filters is assigned to each 
 * pixel is based on a simple pattern consisting of 0 degree, 45 degree,
 * 90 degree and 135 degree linear polarizers.  The pattern used is
 * illustrated below:
 *
 *  0  90  0  90  0  90  0  90
 * 45 135 45 135 45 135 45 135
 *  0  90  0  90  0  90  0  90
 * 45 135 45 135 45 135 45 135
 *
 * The assignment of the relative spectral response and polarization to
 * each pixel is accomplished by cycling through 4 different channel
 * indexes.
 */
int
main( int argCount, char ** argVector )
{
    // the focal length is 100 mm
    const double focalLength = 100.0e-03;

    // the array size is 176 x 144
    const int xArraySize = 176;
    const int yArraySize = 144;

    // the pixel pitch is 80 microns
    const double pixelPitch = 80.0e-06;

    // the pixel IFOV can be computed
    const double pixelIfov = atan( pixelPitch / focalLength );

    // open the output pixel database file
    std::ofstream pixelFile( "pixels.dat" );

    // iterate through all the Y rows in this array
    for( int yy = 0; yy < yArraySize; yy++ ) {

        // iterate through all the X pixels in this row
        for( int xx = 0; xx < xArraySize; xx++ ) {

            // compute the pixel X/Y location
            double xLoc = ( xx - ( xArraySize / 2.0 )) * pixelPitch;
            double yLoc = (( yArraySize / 2.0 ) - yy ) * pixelPitch;

            // compute the pixel X/Y angles
            double xAngle = atan( xLoc / focalLength );
            double yAngle = atan( yLoc / focalLength );

            // figure out what filter it gets
            int channelIndex;
            if(( yy % 2 ) == 0 ) {
                if(( xx % 2 ) == 0 ) {
                    channelIndex = 0;
                }
                else {
                    channelIndex = 1;
                }
            }
            else {
                if(( xx % 2 ) == 0 ) {
                    channelIndex = 2;
                }
                else {
                    channelIndex = 3;
                }
            }

            // write this pixel out to the file
            pixelFile
                << xx << " " << yy << " "
                << xAngle << " " << yAngle << " "
                << pixelIfov << " " << pixelIfov << " "
                << channelIndex << std::endl;
        }
    }
}
